#include "cvManager.h"





cvManager::cvManager(){
	bSetup				= false;
	which				= 0;	
	cornerSensitivity	= 5;
	activityThresh		= 3.8;
}

//changing cameras or switching from/to the camera mode
//requires the app to be restarted - mabe we can change this?
//---------------------------------------------------------------------------------		
void cvManager::setupCamera(int deviceNumber, int width, int height){
	VG.setVerbose(true);
	//VG.setDeviceID(deviceNumber);
	VG.initGrabber(width, height);
	W = VG.width;
	H = VG.height;			
	bCameraSetup = true;
	which = 2;

	setupCV();
}
		
//same as above - from/to using test movies
//requires a restart
//---------------------------------------------------------------------------------		
void cvManager::setupVideo(string videoPath){
	
	VP.loadMovie(videoPath);
	VP.play(); 
	VP.setPosition(0.0);
	VP.setSpeed(0.90);
	VP.setUseTexture(true);
	
	W				= VP.width;
	H				= VP.height;
	bVideoSetup		= true;
	which			= 1;
	
	setupCV();
}

//good for adjusting the color balance, brightness etc
//---------------------------------------------------------------------------------		
void cvManager::openCameraSettings(){
	if(bCameraSetup)VG.videoSettings();			
}

//---------------------------------------------------------------------------------		
void cvManager::useCamera(){
	if(bCameraSetup)which = 2;	
}		

//---------------------------------------------------------------------------------		
void cvManager::useVideo(){
	if(bVideoSetup)which = 1;	
}	


//do all our openCV allocation
//---------------------------------------------------------------------------------------------------------------------------------------		
void cvManager::setupCV(){
	
	if(bSetup) return;
	
	cardAnal.setup(CARD_CV_W, CARD_CV_H);
			
	//our openCV inits
	VideoFrame.allocate(W, H);
	PresenceFrame.allocate(W, H);
	GreyFrame.allocate(W, H);
	blobTrack.setup(W, H, 200);
	ImageFrame.allocate(W,H);
	bSetup = true;
}



void cvManager::passInImageToFindCardsIn(ofImage &imgToLookAt){
	ImageFrame.setFromPixels(imgToLookAt.getPixels(), W, H);
}


//---------------------------------------------------------------------------------------------------------------------------------------		
int cvManager::findCards(card * cards, int threshold, int minSize, int maxSize, bool bUseImageNotVideo){
			
	
	int nCardsToFind = NUM_CARDS;
	
	if (!bUseImageNotVideo){
		
		unsigned char * pixCam;
		
		if(which == 1){
			VP.idleMovie();	
			pixCam 	= VP.getPixels();
		} else if(which == 2){
			VG.grabFrame();
			pixCam 	= VG.getPixels();		
		}else{
			return 0; // no blobs??				
		}
		
		VideoFrame.setFromPixels(pixCam, W, H);
		GreyFrame.setFromColorImage(VideoFrame);
	
	} else {
		
		nCardsToFind = 1; // from images we find 1 
		GreyFrame.setFromColorImage(ImageFrame);
	
	}

	
	// Part 3 - do thresholding	
	PresenceFrame = GreyFrame;	
	PresenceFrame.threshold(threshold);
	//PresenceFrame.erode_3x3();

	
	// Part 4 - find the largest blobs of possible candidates 
	int numBlobs = Contour.findContours(PresenceFrame, minSize, maxSize, 50, false);  
	if( numBlobs == 0) return 0;
	
	
	// Part 5 - do shit with the data
	
	blobTrack.identifyBlobs(Contour.blobs, numBlobs, 20);

				
	
	// Part 6 - For each card we find the four corners and calcuate the hu moments for each card
		
				
	int k = 0;
	for(int i = 0; i < numBlobs; i++){

		if( k >= nCardsToFind ) break;
		
		//some things that will cause us to skip a card
		//if(blobTrack.getBlobActivity(i) > 5 * activityThresh) continue;	 
		if(Contour.blobs[i].area < MIN_CARD_AREA) continue;
		
		//smooth the contour
		vector <ofxVec2f> smoothContour;
		contAna.smoothContour(Contour.blobs[i], smoothContour, 0.1);
		
		//get the number of corners
		int numFound = corner.findCorners(smoothContour, cornerSensitivity);
		
		if(numFound != 4){
			//try and find corners with zach's line detector
			continue; //replace with line detector
		}

		cards[k].setPoints(smoothContour[corner.getTipIndex(0)],  smoothContour[corner.getTipIndex(1)], smoothContour[corner.getTipIndex(2)], smoothContour[corner.getTipIndex(3)]);
		cardGeom.calculateGeometry(cards[k]);
		
		//cardAnal.getCardAsCvImage(cards[k],  GreyFrame);
		cardAnal.getCardAsCvImage(cards[k], GreyFrame, bUseImageNotVideo ? ImageFrame : VideoFrame); 
		cardAnal.dynamicThreshold();
		
		// could be useful for pixel pixel stuff :
		//if (bUseImageNotVideo){
		
		// get my cards 
		cards[k].myCardImage.setFromPixels(cardAnal.cardImg.getPixels(), CARD_CV_W, CARD_CV_H, OF_IMAGE_GRAYSCALE);
		cardAnal.cardImg.mirror(true, true);
		cards[k].myCardImageFlipped.setFromPixels(cardAnal.cardImg.getPixels(), CARD_CV_W, CARD_CV_H, OF_IMAGE_GRAYSCALE);
		
		//}
		
		cardAnal.calculateHuMoments(cards[k], true);
		cardAnal.calculateCardProperties(cards[k]);
		k++;
	}
	
	
	// Part 8 - we now return how many cards we found
	int numLegitCards = k;
	return numLegitCards;			
} 


//---------------------------------------------------------------------------------				
void cvManager::resetBlobID(){
	blobTrack.resetCounter();
}
		
//---------------------------------------------------------------------------------				
void cvManager::toggleVideoPlayback(){
	if(VP.getSpeed() == 0.0){
		VP.setSpeed(1.0);
	}else{
		VP.setSpeed(0.0);
	}
}

		
//---------------------------------------------------------------------------------				
void cvManager::draw(float x, float y){
	ofSetColor(255, 255, 255);
	PresenceFrame.draw(x, y);
}



