#include "testApp.h"

static bool toggle = false;

//--------------------------------------------------------------
void testApp::setup(){	 
	
	useVideo  = true;
	threshold = 90;
	blob_min  = 30;
	blob_max  = 3000;
	setPct	  = false;
	videoPct  = 1.0;
	
	GUI.setTitle("cv settings - space to hide");   //<-- set the title of the gui 
	GUI.addToggle("use_video", &useVideo);
	GUI.addSlider("cv_threshold", &threshold, 0, 255);  
	GUI.addSlider("cv_blob_min", &blob_min, 0, 0.2 * 320 * 240); 
	GUI.addSlider("cv_blob_max", &blob_max, 0, 320 * 240);  

	GUI.setAutoSave(true);     // <-- enable auto saving 
	GUI.setFromXML("cv_settings.xml");  //<-- set from xml 
	GUI.setVerbose(true);      //<-- talk to me 
	
	GUI.setGuiPos(660, 90);
	
	//GUI.toggleRender();

	// VARIOUS APP SETTINGS
	//set background to black
	ofBackground(30, 30, 30);	
	ofSetVerticalSync(true);	
	ofSetWindowTitle("card play");
	ofSetCircleResolution(50);
	
	//SETUP OUR VIDEO SOURCE
	if(!useVideo){
		videoCV.setupCamera(2, 640, 480);
	}else{
		videoCV.setupVideo("videos/FiftyTwo.mov");
		GUI.addMovieSlider("video slider", &videoCV.VP);
	}

	cardMan.setup("card_bank.xml");	
	whichCard = 1;	
	
	
	for (int i = 0; i < 52; i++){
		videoCV.passInImageToFindCardsIn(cardMan.cardImages[i]);
		videoCV.findCards(cardMan.newCardBank + i, threshold, blob_min, blob_max, true);
	}
	
	//std::exit(0);
}

//--------------------------------------------------------------
void testApp::update(){

	if(!useVideo)videoCV.useVideo();	
	else videoCV.useCamera();
	
	cardMan.resetCards();
	numFound = videoCV.findCards(cardMan.cards, threshold, blob_min, blob_max);
	
	for(int i = 0; i < NUM_CARDS; i++){
		if( cardMan.cards[i].good ){
			// zach 
			cardMan.identifyCardSimple(cardMan.cards[i], 1);			
			cardMan.identifyCardViaHu(cardMan.cards[i], 1);
			cardMan.identifyCardViaPixels(cardMan.cards[i]);
			//
		}
	}		
}

//--------------------------------------------------------------
void testApp::draw(){


	ofSetColor(0xFFFFFF);	
	videoCV.draw(0,0);
	cardMan.draw(0, 0);

	ofSetColor(0xFFFFFF);
	videoCV.cardAnal.cardImg.draw(videoCV.W, 0);
	videoCV.cardAnal.thresholdedFrame.draw(videoCV.W + CARD_CV_W, 0);

	ofSetColor(0xFFFFFF);
	string info = "framerate: ";
	info += ofToString(ofGetFrameRate()) + "\n";
	info += "numFound: "+ofToString(numFound) + "\n";
	info += "whichCard: "+ofToString(whichCard) + "\n";
	
	ofDrawBitmapString(info, 10, 15);
	
	/*
	ofSetColor(255, 255, 255);
	for (int i = 0; i < 52; i++){
		if (ofGetFrameNum() % 10 < 5)  cardMan.newCardBank[i].myCardImage.draw((i%13)*64,(i/13)*80);
		else cardMan.newCardBank[i].myCardImageFlipped.draw((i%13)*64,(i/13)*80);
	}
	
	for (int i = 0; i < 52; i++){
		int x1 = (i%13)*64;
		int y1 = (i/13)*80;
		int x2 = x1 + CARD_CV_W;
		int y2 = y1 + CARD_CV_H;
		
		if (mouseX > x1 && mouseX < x2){
			if (mouseY > y1 && mouseY < y2){
				cardMan.newCardBank[i].drawAnalysisResults(x1 + 30, y1 + 30);
			}
		}
	}*/

}


//--------------------------------------------------------------
void testApp::keyPressed  (int key){ 
	
	//train the first 9 cards (1 - 9)
	if( key - 48 >= 1 && key - 48 <= 9 ){
		for(int i = 0; i < NUM_CARDS; i++){
			if( cardMan.cards[i].good )cardMan.saveCardAs(cardMan.cards[i], key - 48);
		}
	}
	
	//set card to store as next card
	if(key == OF_KEY_RIGHT){
		whichCard++;
		whichCard %= 53;
	}
	
	//set card to store as previous card
	if( key == OF_KEY_LEFT){
		whichCard--;
		if(whichCard <= 0)whichCard = 1;
	}
	
	//save card to xml and increment
	//we have to go through all the cards - but there should really only be one that is active when saving
	if(key == 'z'){
		for(int i = 0; i < NUM_CARDS; i++){
			if( cardMan.cards[i].good )cardMan.saveCardAs(cardMan.cards[i], whichCard);
		}
		whichCard++;
	}
	
	if(key == 's'){
		GUI.saveToXML("cv_settings.xml");   
	}else if(key == 'r'){
		GUI.setFromXML("cv_settings.xml"); 
	}
	
	if(key == ' '){
		GUI.toggleRender();
	}
	
	 else if(key == 'c'){
		videoCV.openCameraSettings();
	 }
}

//--------------------------------------------------------------
void testApp::keyReleased (int key){ 

}

//--------------------------------------------------------------
void testApp::mouseMoved(int x, int y ){		
	
}

//---------------------------------------------------------------
void testApp::mouseDragged( int x, int y, int button){

}

//---------------------------------------------------------------
void testApp::mousePressed( int x, int y, int button ){

}

//---------------------------------------------------------------
void testApp::mouseReleased(){

}


//--------------------------------------------------------------
void testApp::loadSettings(){

}

//--------------------------------------------------------------
void testApp::saveSettings(){

}
